/*
 * Copyright (C) 2005-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "s4jproxy.h"
#include "logutils.h"
#include "client/Sync4jClient.h"
#include "base/Log.h"

#include "pim/maincpp.h"
#include "pim/SettingFunctions.h"
#include "base/startcmd.h"
#include "spdm/DMTree.h"
#include "spdm/DMTreefactory.h"

static wchar_t *getSourceName(const wchar_t *root, const wchar_t *uri)
{
    int namelen = wcslen(root) + wcslen(TEXT(CONTEXT_SPDS_SOURCES)) + 5;
    int i = 0;
    wchar_t *nodeName = new wchar_t[namelen];
    wchar_t *ret = NULL;

    if(!nodeName)
        return NULL;

    wsprintf(nodeName, TEXT("%s/%s"), root, TEXT(CONTEXT_SPDS_SOURCES));

	DMTree* dmt = DMTreeFactory::getDMTree(root);

    ManagementNode *node = dmt->getManagementNode(nodeName);
    if ( ! node ) {
        lastErrorCode = ERR_INVALID_CONTEXT;
        wsprintf(lastErrorMsg, ERRMSG_INVALID_CONTEXT, nodeName);
        DebugPrint(TEXT("getSourceName: error: %s"), lastErrorMsg);
        goto finally;
    }
    for(i=0; i<node->getChildrenMaxCount(); i++) {
        ManagementNode *src = node->getChild(i);
        wchar_t *tmp = src->getPropertyValue(PROPERTY_SOURCE_URI);
        //DebugPrint(TEXT("i:%d uri:%s tmp:%s\n"), i, uri, tmp);
        if (wcscmp(uri, tmp) == 0) {
            ret = src->getPropertyValue(PROPERTY_SOURCE_NAME);
            break;
        }   
        delete [] tmp;
    }
    
finally:
    if (dmt) {
        delete dmt;
    }
    if (node) {
        delete node;
    }
    return ret;
}

// Default URI
// FIXME: put it in a common header
#define APPLICATION_URI  TEXT("Funambol/SyncclientPIM")

// Singleton instance
S4JProxy * S4JProxy::instance = 0;

//-------------------------------------------------------------- Static functions
#define ASK_MESSAGE TEXT("The server has notified that new data is available. Do you want to synchronize now?")

static bool askConfirm()
{   
    //
    //insert popup
    //
    wchar_t buf[128];
    getClientConfigurationInternal (TEXT(""), PROPERTY_SPDM_PUSH_NOTIFICATION, 
                                     buf, FALSE);
    //
    // possible value 1 and 2. 
    // 1 is download auto
    // 2 is ask user
    //
    if (wcscmp(buf, TEXT("2")) == 0) {
        
        DebugPrint(TEXT("Confirm needed: user alerted.\n"));

        int val = MessageBox(NULL, ASK_MESSAGE, TEXT("Alert"),
                             MB_YESNO | MB_ICONQUESTION);

        if (val == IDYES) {
            DebugPrint(TEXT("User said yes.\n"));
            return true;
        }
        else {
            DebugPrint(TEXT("User said no.\n"));
            return false;
        }
    }
    DebugPrint(TEXT("Confirm not requested, sync auto.\n"));
    return true;
}

// Debug log.
static void PrintMessage(SyncNotification& np)
{
    DebugPrint(L"Version:\t%d\n", np.getVersion());
    DebugPrint(L"UIMode:\t%d\n", np.getUIMode());
    DebugPrint(L"Initiator:\t%S\n", 
            ( np.getInitiator() == UserInitiated )
            ? "UserInitiated" : "ServerInitiated" );
    DebugPrint(L"Session ID:\t%d (%x)\n",
            np.getSessionId(), np.getSessionId() );
    DebugPrint(L"Server ID:\t%s\n", np.getServerId() );
    DebugPrint(L"NumSyncs:\t%d\n", np.getNumSyncs() );

    for (int i=0; i<np.getNumSyncs(); i++){
        SyncAlert *sync;

        sync=np.getSyncAlert(i);

        if( ! sync ){
            DebugPrint(L"Error on sync #%d\n", i);
        }
        DebugPrint(L"Sync type:\t%d\n", sync->getSyncType() );
        DebugPrint(L"Content type:\t%d\n", sync->getContentType() );
        DebugPrint(L"Server URI:\t%s\n", sync->getServerURI() );
    }
}


S4JProxy* S4JProxy::getInstance()
{
    if( !instance )
        instance=new S4JProxy();

    return instance;
}

void S4JProxy::dispose()
{
    if( instance )
        delete instance;
}
        
S4JProxy::S4JProxy()
{
	errcode=SNErr_Undefined;
	syncRet=0;
	syncMessage=0;
}

S4JProxy::~S4JProxy()
{
	if(syncMessage) 
		delete [] syncMessage;
}

S4JProxy::RetCode S4JProxy::parsePkg0(const char *msg, int msglen)
{
	DebugPrint(TEXT("calling parse\n"));
    errcode=np.parse(msg,msglen);
    PrintMessage(np);
	switch (errcode) {
        case SNErr_Ok :
            //PrintMessage(np);
            return Ok;                  // OK
        case SNErr_Incomplete :
            return Continue;            // Get more bytes
        default:
            return Error;               // Error
    }
}

const char *S4JProxy::getResponse()
{
    switch(errcode){
        case SNErr_Ok:
            return RESP_OK;
        case SNErr_BadReq:
            return RESP_BADREQ;
        case SNErr_InvSync:
            return RESP_INVSYNC;
        case SNErr_Incomplete:
            return "400 - Message incomplete\n";
        case SNErr_NullValue:
            return RESP_NULLP;
        default:
            return "500 - Internal error\n";
    }
}

int S4JProxy::sync(bool debug)
{
#ifdef DEBUG
    // XXX Debug code
    if (debug){
        //- Generated with xxd --------------------------------------------------
        unsigned char message_bin[] = {
          0x1d, 0xca, 0x85, 0xbf, 0x59, 0x2e, 0xff, 0x5d, 0x53, 0xf6, 0xee, 0xff,
          0x57, 0x3d, 0xc1, 0x36, 0x03, 0x18, 0x00, 0x00, 0x00, 0x00, 0x01, 0x0f,
          0x73, 0x79, 0x6e, 0x63, 0x2e, 0x73, 0x65, 0x72, 0x76, 0x65, 0x72, 0x2e,
          0x63, 0x6f, 0x6d, 0x10, 0x60, 0x00, 0x00, 0x07, 0x07, 0x63, 0x6f, 0x6e,
          0x74, 0x61, 0x63, 0x74
        };
        unsigned int message_bin_len = 52;
        //-----------------------------------------------------------------------
        errcode=np.parse((char *)message_bin, message_bin_len);
        if(errcode){
            fprintf(stderr,"Error number: %d\n", errcode);
            return errcode;
        }
    }
#endif

    // No sync info from server?
    if (np.getNumSyncs() == 0) {
        LOG.error(TEXT("No server alert info.\n"));
        return -1;
    }

    wchar_t cmdline[MAX_PATH] = TEXT("once ");

    int count=0;
    for (int i=0; i<np.getNumSyncs(); i++){
        SyncAlert *sync;
        wchar_t *srcname;

        sync=np.getSyncAlert(i);

        if( !sync ){
            wchar_t dbg[256];
            wsprintf(dbg, TEXT("Error on sync #%d\n"), i);
            LOG.error(dbg);
        }

        srcname = getSourceName(APPLICATION_URI, sync->getServerURI());
        if (srcname) {
            // Add separator, if not first
            if (count>0) 
                wcscat( cmdline, TEXT(",") );
            wcscat( cmdline, srcname );
            count ++;
            delete [] srcname;
        }
    }

    if (askConfirm()) {
        DebugPrint( TEXT("Starting: %s %s"), SYNCAPP, cmdline );
        if(startcmd(SYNCAPP, cmdline))
            DebugPrint( TEXT("Sync process started.\n") );
        else
            DebugPrint( TEXT("Error starting sync process.\n") );
    }

	return syncRet;
}

const wchar_t *S4JProxy::getSyncMessage() {
    return syncMessage;
}
        
bool S4JProxy::hasErrors() {
    return errcode != SNErr_Ok;
}


